from PIL import Image

def get_greenscreen_area_details(image_filename: str):
    """
    This methods detects the greenscreen area in the provided
    'image_filename' image that contains one and valid 
    greenscreen.

    This method returns a dictionary containing 'rgb_color', 
    that is the green rgb color of the greenscreen area, and
    'upper_left_pixel' and 'lower_right_pixel', that are the
    greenscreen area corners we need to know. Each of those
    are (x, y) tuples containing the pixel coordinates. The
    'rgb_color' will be a (r, g, b) tuple containing the 
    values from 0 to 255 for each color.

    This method will return None in 'rgb_color' if no green
    color detected. This method will fail if more than one
    separated greenscreens in the provided 'image_filename'.
    """
    if not image_filename:
        return None
    
    green_rgb_color = get_most_common_rgb_color(image_filename)

    image = Image.open(image_filename).convert('RGB')

    upper_left = {
        'x': 99999,
        'y': 99999,
    }
    lower_right = {
        'x': -1,
        'y': -1,
    }

    for x in range(image.width):
        for y in range(image.height):
            rgb_color = (r, g, b) = image.getpixel((x, y))

            if rgb_color == green_rgb_color:
                if x < upper_left['x']:
                    upper_left['x'] = x
                if y < upper_left['y']:
                    upper_left['y'] = y
                
                if x > lower_right['x']:
                    lower_right['x'] = x
                if y > lower_right['y']:
                    lower_right['y'] = y

    # We apply some margin to make sure we fit the green screen
    MARGIN = 2

    if (upper_left['x'] - MARGIN) > 0:
        upper_left['x'] -= MARGIN
    else:
        upper_left['x'] = 0

    if (upper_left['y'] - MARGIN) > 0:
        upper_left['y'] -= MARGIN
    else:
        upper_left['y'] = 0

    if (lower_right['x'] + MARGIN) < 1920:
        lower_right['x'] += MARGIN
    else:
        lower_right['x'] = 1920

    if (lower_right['y'] + MARGIN) < 1080:
        lower_right['y'] += MARGIN
    else:
        lower_right['y'] = 1080

    return {
        'rgb_color': green_rgb_color,
        'upper_left_pixel': (upper_left['x'], upper_left['y']),
        'lower_right_pixel': (lower_right['x'], lower_right['y'])
    }

def get_most_common_rgb_color(image_filename, force_green = True):
    """
    Returns the most common green rgb color that exist in the provided
    'image_filename'. There could be no green color so it will return
    None, or the green color as (r, g, b) if existing.

    # TODO: This method could be made more generic by using a range
    # and then specializing this one by using that with green range
    """
    colors = {}
    image = Image.open(image_filename).convert('RGB')

    # We will check the most common rgb color (should be the green of mask)
    for x in range(image.width):
        for y in range(image.height):
            rgb_color = (r, g, b) = image.getpixel((x, y))

            if not rgb_color in colors:
                colors[rgb_color] = 1
            else:
                colors[rgb_color] += 1

    # Check which one is the most common
    most_used_rgb_color = {
        'color': None,
        'times': 0,
    }
    for key, value in colors.items():
        if force_green:
            # We only care about green colors
            r, g, b = key
            is_green = (r >= 0 and r <= 100) and (g >= 100 and g <= 255) and (b >= 0 and b <= 100)
            if is_green:
                if value > most_used_rgb_color['times']:
                    most_used_rgb_color = {
                        'color': key,
                        'times': value
                    }
        else:
            if value > most_used_rgb_color['times']:
                most_used_rgb_color = {
                    'color': key,
                    'times': value
                }

    return most_used_rgb_color['color']


from moviepy.editor import VideoFileClip
from yta_general_utils.type_checker import variable_is_type
from yta_general_utils.file_processor import file_is_video_file
from typing import Union

def extract_all_frames_from_video(video_input: Union[VideoFileClip, str], output_folder: str):
    """
    This method will extract all the 'video_input' frames in the
    provided 'output_folder' with the 'frameXXXXX.png' name, starting
    from 0 to the last frame.
    """
    if not video_input:
        return None
    
    if not output_folder:
        return None
    
    if not output_folder.endswith('/'):
        output_folder += '/'
    
    if variable_is_type(video_input, str):
        if not file_is_video_file(video_input):
            return None
        
        video_input = VideoFileClip(video_input)

    video_input.write_images_sequence(output_folder + 'frame%05d.png')

    return output_folder

def extract_frames_from_video(video_input: Union[VideoFileClip, str], frames_number: int, output_folder: str):
    """
    This method will extract only 'frames_number' frames of the 
    provided 'video_input' video and will be stored in also
    provided 'output_folder' with the 'frameXXXXX.png' name, 
    starting from 0 to the last frame.

    This method will split the video in 'frames_number' segments
    of the same duration and will extract the frame in the middle
    of each of those segments.

    To clarify, if a video lasts 20 seconds and the 'frames_number'
    parameter is 4, this method will extract the 4 frames that are
    in '2.5s', '7.5s', '12.5s' and '17.5s' time moments.
    """
    if not video_input:
        return None
    
    if not output_folder:
        return None
    
    if frames_number < 1:
        return None
    
    if not output_folder.endswith('/'):
        output_folder += '/'
    
    if variable_is_type(video_input, str):
        if not file_is_video_file(video_input):
            return None
        
        video_input = VideoFileClip(video_input)

    total_video_frames = video_input.fps * video_input.duration
    if frames_number > total_video_frames:
        return None

    # We will split the video in half 'frames_number' + 1 times
    # 20s video -> 4 frames => 20/4 = 5s*i + (5s/2) => 
    # i = 0,    i = 1,  i = 2,  i = 3
    # 2.5s,     7.5s,   12.5s,  17.5s
    time_fraction = video_input.duration / frames_number # 5s
    time_fraction_half = time_fraction / 2 # 2.5s
    moments = []

    for i in range(frames_number):
        moments.append(time_fraction * i + time_fraction_half)

    for index, moment in enumerate(moments):
        output_filename = output_folder + 'frame' + (str(index) + '').zfill(5) + '.png'
        video_input.save_frame(t = moment, filename = output_filename)

    return output_folder

def extract_frame_from_video(video_input: Union[VideoFileClip, str], time: float, output_filename: str):
    """
    Extracts a frame from the provided 'video_input' that is exactly at the
    also provided 'time' moment. The frame will be stored as 
    'output_filename'.
    """
    if not video_input:
        return None
    
    if not output_filename:
        return None
    
    if time < 0:
        return None
    
    if variable_is_type(video_input, str):
        if not file_is_video_file(video_input):
            return None
        
        video_input = VideoFileClip(video_input)

    if time > video_input.duration:
        return None

    video_input.save_frame(t = time, filename = output_filename)

    return output_filename


from yta_multimedia.video.generation.manim.constants import SCENE_HEIGHT, SCENE_WIDTH

def width_to_manim_width(width):
    """
    You provide a real width in pixels (maybe 1920) and it is returned in the
    manim equivalent width. We consider a 16:9 proportion and 1920 as the
    maximum valid width.

    This method is built by myself to work better with a 16:9 proportion of 
    1920x1080 pixels. The manim system is different, so I made this to
    simplify the process.
    """
    return (width * SCENE_WIDTH) / 1920

def manim_width_to_width(width):
    """
    TODO: Write help
    """
    return (width * 1920) / SCENE_WIDTH

def height_to_manim_height(height):
    """
    You provide a real height in pixels (maybe 1080) and it is returned in the
    manim equivalent pixels. We consider 16:9 proportion and 1080 as the 
    maximum valid height.

    This method is built by myself to work better with a 16:9 proportion of 
    1920x1080 pixels. The manim system is different, so I made this to
    simplify the process.
    """
    return (height * SCENE_HEIGHT) / 1080

def manim_height_to_height(height):
    """
    TODO: Write help
    """
    return (height * 1080) / SCENE_HEIGHT